import { Loader, type ResolvedAsset, type SpritesheetData, type SpritesheetFrameData, Texture } from 'pixi.js';
import { type HostConfig } from './typedefs/HostConfig';
import { type LDTKEntity } from './typedefs/LDTKEntity';
import { type LDTKLevel } from './typedefs/LDTKLevel';
import { type LDTKLevelLayer } from './typedefs/LDTKLevelLayer';
import { type LDTKLevelLayerTile } from './typedefs/LDTKLevelLayerTile';
import { type LDTKObject } from './typedefs/LDTKObject';
import { type LDTKTileset } from './typedefs/LDTKTileset';
import { type ParsedTilesetMap } from './typedefs/ParsedTilesetMap';
/** The root of all parsers. */
export declare abstract class BaseParser<Config extends HostConfig = HostConfig> {
    /**
     * Generates a base object for sprite atlases.
     *
     * @param imageSrc The path to the image file.
     * @param pixelHeight The pixel height of tiles in the tileset.
     * @param pixelWidth The pixel width of tiles in the tileset.
     * @returns The generated atlas base.
     */
    generateBaseAtlas(imageSrc: string, pixelHeight: number, pixelWidth: number): SpritesheetData;
    /**
     * Generates a base tileset object.
     *
     * @param columnCount The number of columns in the tileset.
     * @param id The ID of the tileset.
     * @param rowCount The number of rows in the tileset.
     * @param tileGridHeight The height of columns in the tileset.
     * @param tileGridWidth The width of a tile grid cell.
     * @returns The generated tileset base.
     */
    generateBaseTileset(columnCount: number, id: number, rowCount: number, tileGridHeight: number, tileGridWidth: number): LDTKTileset;
    /**
     * Creates an atlas frame from Ldtk tile data.
     *
     * @param column The grid-relative column of the tile.
     * @param row The grid-relative row of the tile.
     * @param tileGridSize The size of a tile grid cell.
     * @returns The generated tile frame data.
     */
    generateTileFrameData(column: number, row: number, tileGridSize: number): SpritesheetFrameData;
    /**
     * Retrieves the texture for a tile from a tileset.
     *
     * @param tileID The ID of the tile.
     * @param tileset The tileset to retrieve the texture from.
     * @returns The tile texture.
     */
    getTileTextureFromTileset(tileID: number | string, tileset: LDTKTileset): Texture;
    /**
     * Parses a .ldtk file.
     *
     * @param asset The LDtk file asset.
     * @param ldtkString The string represnetation of the LDtk file contents.
     * @param loader The loader being used to load this asset.
     * @returns The parsed LDtk object.
     */
    parse(asset: ResolvedAsset, ldtkString: string, loader: Loader): Promise<LDTKObject<Config["LDTKProject"]>>;
    /**
     * Parses entities from LDtk source data.
     *
     * @param sourceData Entity data from LDtk source.
     * @returns An array of parsed entities.
     */
    parseEntities(sourceData: Config['EntityInstance'][]): LDTKEntity[];
    /**
     * Parses a layer from LDtk source data.
     *
     * @param sourceData Layer data from LDtk source.
     * @param tilesets Tilesets that have been loaded from the LDtk source.
     * @returns The parsed layer.
     */
    parseLayer(sourceData: Config['LayerInstance'], tilesets: ParsedTilesetMap): LDTKLevelLayer;
    /**
     * Parses a level from LDtk source data.
     *
     * @param sourceData Level data from the LDtk source.
     * @param tilesets Tilesets that have been loaded from the LDtk source.
     * @returns The parsed level.
     */
    parseLayers(sourceData: (Config['LayerInstance'] | null)[], tilesets: ParsedTilesetMap): LDTKLevelLayer[];
    /**
     * Parses a level from LDtk source data.
     *
     * @param assetBasePath The base path from which the level will be loaded.
     * @param sourceData Level data from the LDtk source.
     * @param tilesets Tilesets that have been loaded from the LDtk source.
     * @returns The parsed level.
     */
    parseLevel(assetBasePath: string, sourceData: Config['Level'], tilesets: ParsedTilesetMap): Promise<LDTKLevel>;
    /**
     * Retrieves and parses levels from the LDtk source data.
     *
     * @param assetBasePath The base path from which levels will be loaded.
     * @param sourceData LDtk source data.
     * @param tilesets Tilesets that have already been loaded from the LDtk source.
     * @returns An array of parsed levels.
     */
    parseLevels(assetBasePath: string, sourceData: Config['Level'][], tilesets: ParsedTilesetMap): Promise<LDTKLevel[]>;
    /**
     * Parses tiles from LDtk source data.
     *
     * @param sourceData Tile data from LDtk source.
     * @param tileset The layer's tileset.
     * @returns An array of parsed tiles.
     */
    parseTiles(sourceData: Config['TileInstance'][], tileset: LDTKTileset): LDTKLevelLayerTile[];
    /**
     * Parses a tileset from LDtk source data.
     *
     * @param assetBasePath The base path from which the tileset will be loaded.
     * @param loader The Pixi loader.
     * @param sourceData Tileset source data.
     * @returns The parsed tileset.
     */
    parseTileset(assetBasePath: string, loader: Loader, sourceData: Config['TilesetDefinition']): Promise<LDTKTileset | null>;
    /**
     * Retrieves and parses tilesets from LDtk source data.
     *
     * @param assetBasePath The base path from which tilesets will be loaded.
     * @param loader The Pixi loader.
     * @param sourceData LDtk source data.
     * @returns A map of tilesets.
     */
    parseTilesets(assetBasePath: string, loader: Loader, sourceData: Config['TilesetDefinition'][]): Promise<ParsedTilesetMap>;
    /** Converts a string to an LDtk project. */
    abstract toLDTKProject(ldtkString: string): Config['LDTKProject'];
}
//# sourceMappingURL=BaseParser.d.ts.map